(set-dynamic-constraint-checking TRUE)
(defmodule FISH (export deftemplate menu) )
(defmodule WEATHER (import FISH deftemplate menu) (export deftemplate acceptable_condition))
(defmodule TRIP (import FISH deftemplate menu) (import WEATHER deftemplate acceptable_condition))
(focus WEATHER FISH TRIP )
(get-focus-stack)
(run)
(pop-focus)
(run)
(pop-focus)
(run)	
(pop-focus)	
(run)

(deftemplate FISH::fish
	(slot species )
	(multislot habitat )
	(slot seasonal_availability ) ;date of fish runs
	(slot open_season ) ;the date legal open season begins
	(slot close_season ) ;the date the legal season closes
	(slot size_min ) ;minimum legal size allowed to be taken
	(slot limit ) ;legal maximum allowed number of fish allowed
	(slot fishing_type ) ;bottom or gamefishing 
	(slot fishing_location ) ;inshore or offshore 
	(multislot best_trip ) ; best to fish day or night
	(slot desireability) ; how much people like to fish
)

(deftemplate FISH::menu 
 (slot species)
 (multislot habitat )
 (slot seasonal_availability ) ;date of fish runs
 (slot size_min ) ;minimum legal size allowed to be taken
 (slot limit ) ;legal maximum allowed number of fish allowed
 (slot fishing_type ) ;bottom or gamefishing 
 (slot fishing_location ) ;inshore or offshore 
 (multislot best_trip ) ; best to fish day or night
 (slot desireability) ; how much people like to fish
 (slot bonus (type SYMBOL) (allowed-symbols ON OFF) ) ; how much people like to fish
)

(deftemplate FISH::season
	(slot description ) (slot start_date) (slot end_date)
)

(deftemplate FISH::date
    (slot categ (type SYMBOL) (allowed-symbols today start_date end_date) )
	 (slot day (type INTEGER) (range 1 31))
	 (slot month (type INTEGER) (range 1 12))
	 (slot dateint (type INTEGER)(range 20101 21231 )) ; (dateint  - 20000 ) is the date
)

(deffunction FISH::openseason (?start ?end ?today ) 
 	(if (> ?start ?end )
	 then
	  (or 
			(and (>= ?today ?start ) (<= ?today  21231)) 
			(and (>= ?today 20101 ) (<= ?today ?end))
		)
	  else (and (>= ?today ?start) (<= ?today ?end))
	)
)

(deffacts fishing_seasons
	( season (description spring_bass_run) (start_date  20401) (end_date 20515) )
	( season (description fall_bass_run) (start_date 21001) (end_date  21201) )
	( season (description winter) (start_date  21215) (end_date 20315) )
	( season (description winter_tautog) (start_date  21015) (end_date 20315) )
	( season (description spring_tautog ) (start_date 20401) (end_date 20515) )
	( season (description warm_months ) (start_date 20415) (end_date 21101) )
)

(deffacts fishes
	(fish (species tautog) (habitat bottom wrecks) (seasonal_availability winter_tautog ) 
	      (open_season 21022) (close_season 21231 ) (size_min 24 ) 
			(limit 4 ) (fishing_type bottom ) (fishing_location offshore ) 
			(best_trip day ) (desireability 5)
	)
	(fish (species tautog) (habitat bottom wrecks) (seasonal_availability winter_tautog ) 
	      (open_season 21022) (close_season 21231 ) (size_min 24 ) 
			(limit 4 ) (fishing_type bottom ) (fishing_location inshore ) 
			(best_trip day ) (desireability 2)
	)
	(fish (species striped_bass) (habitat inshore current rocks ) (seasonal_availability spring_bass_run ) 
	      (open_season 20101 ) (close_season 21231 ) (size_min 31  ) 
			(limit 1 ) (fishing_type game) (fishing_location inshore ) 
			(best_trip day night ) (desireability 4)
	)
	(fish (species striped_bass) (habitat inshore current rocks ) (seasonal_availability fall_bass_run ) 
	      (open_season 20101) (close_season 21231 ) (size_min 31 ) 
			(limit 1 ) (fishing_type game ) (fishing_location inshore ) 
			(best_trip day night ) (desireability 4)
	)
	(fish (species scup) (habitat bottom) (seasonal_availability warm_months ) 
	      (open_season 20501 ) (close_season 20831 ) (size_min 10 ) 
			(limit 30 ) (fishing_type bottom ) (fishing_location inshore ) 
			(best_trip day ) (desireability 3 )
	)
	(fish (species scup) (habitat bottom) (seasonal_availability warm_months ) 
	      (open_season 20501 ) (close_season 20831 ) (size_min 10 ) 
			(limit 30 ) (fishing_type bottom ) (fishing_location offshore ) 
			(best_trip day ) (desireability 4 )
	)
	(fish (species scup) (habitat bottom) (seasonal_availability warm_months ) 
	      (open_season 20901 ) (close_season 21031 ) (size_min 10 ) 
			(limit 45 ) (fishing_type bottom ) (fishing_location offshore ) 
			(best_trip day night) (desireability 4 )
	)
	(fish (species scup) (habitat bottom) (seasonal_availability warm_months ) 
	      (open_season 20901 ) (close_season 21031 ) (size_min 10 ) 
			(limit 45 ) (fishing_type bottom ) (fishing_location inshore ) 
			(best_trip day night) (desireability 3 )
	)
	(fish (species scup) (habitat bottom) (seasonal_availability warm_months ) 
	      (open_season 21101 ) (close_season 21231 ) (size_min 10 ) 
			(limit 30 ) (fishing_type bottom ) (fishing_location inshore ) 
			(best_trip day ) (desireability 3 )
	)
	(fish (species scup) (habitat bottom) (seasonal_availability warm_months ) 
	      (open_season 21101 ) (close_season 21231 ) (size_min 10 ) 
			(limit 30 ) (fishing_type bottom ) (fishing_location offshore ) 
			(best_trip day ) (desireability 4 )
	)
	(fish (species fluke ) (habitat bottom ) (seasonal_availability warm_months) 
	      (open_season 20517) (close_season 20921 ) (size_min 18 ) 
			(limit 5 ) (fishing_type bottom ) (fishing_location inshore ) 
			(best_trip day ) (desireability 3)
	)
	(fish (species bluefish) (habitat open_water current ) (seasonal_availability all_year) 
	      (open_season 20101 ) (close_season 21231 ) (size_min 0 ) 
			(limit 500 ) (fishing_type game ) (fishing_location inshore ) 
			(best_trip day ) (desireability 2 )
	)
	(fish (species bluefish) (habitat open_water current ) (seasonal_availability all_year) 
	      (open_season 20101 ) (close_season 21231 ) (size_min 12 ) 
			(limit 15 ) (fishing_type game ) (fishing_location offshore ) 
			(best_trip day ) (desireability 3 )
	)
	(fish (species cod ) (habitat open_water) (seasonal_availability winter ) 
	      (open_season 20101 ) (close_season 21231 ) (size_min 22 ) 
			(limit 10 ) (fishing_type bottom ) (fishing_location offshore ) 
			(best_trip day ) (desireability 3)
	)
	(fish (species black_sea_bass ) (habitat bottom ) (seasonal_availability warm_months ) 
	      (open_season 20715) (close_season 21031 ) (size_min 14 ) 
			(limit 8 ) (fishing_type bottom ) (fishing_location inshore ) 
			(best_trip day ) (desireability 3 )
	)
	(fish (species black_sea_bass ) (habitat bottom ) (seasonal_availability warm_months ) 
	      (open_season 21101) (close_season 21231 ) (size_min 14 ) 
			(limit 10 ) (fishing_type bottom ) (fishing_location inshore ) 
			(best_trip day ) (desireability 3 )
	)
	(fish (species black_sea_bass ) (habitat bottom ) (seasonal_availability warm_months ) 
	      (open_season 20715) (close_season 21031 ) (size_min 14 ) 
			(limit 8 ) (fishing_type bottom ) (fishing_location offshore ) 
			(best_trip day ) (desireability 3 )
	)
	(fish (species black_sea_bass ) (habitat bottom ) (seasonal_availability warm_months ) 
	      (open_season 21101) (close_season 21231 ) (size_min 14 ) 
			(limit 10 ) (fishing_type bottom ) (fishing_location offshore ) 
			(best_trip day ) (desireability 3 )
	)
	(fish (species black_sea_bass_federal ) (habitat bottom ) (seasonal_availability warm_months ) 
	      (open_season 20515) (close_season 20921 ) (size_min 14 ) 
			(limit 10 ) (fishing_type bottom ) (fishing_location offshore_federal_waters ) 
			(best_trip day ) (desireability 3 )
	)
	(fish (species black_sea_bass_federal ) (habitat bottom ) (seasonal_availability warm_months ) 
	      (open_season 21022) (close_season 21231 ) (size_min 14 ) 
			(limit 10 ) (fishing_type bottom ) (fishing_location offshore_federal_waters ) 
			(best_trip day ) (desireability 3 )
	)
	(fish (species ling ) (habitat mud bottom ) (seasonal_availability winter ) 
	      (open_season 20101) (close_season 21231 ) (size_min  0) 
			(limit 500 ) (fishing_type bottom ) (fishing_location offshore ) 
			(best_trip day ) (desireability 2)
	)
)

(FISH::defrule getdate
	=>
 ( printout t "Enter the Date" crlf "What is the Month (as an integer):" crlf )
 (bind ?month (read) )
 (printout t "What is the Day (as an integer): " crlf)
 (bind ?day (read) )
 (bind ?day2 (+ ?day  10000))
 (bind ?month2 (+ (* ?month  100) 10000))
 (assert (date (categ today) (month ?month) (day ?day) (dateint (+ ?month2 ?day2))))
) 

(FISH::defrule whatsonthemenu 
 (fish (species ?sp) (open_season ?open) (close_season ?close)
       (habitat $?hb) (seasonal_availability ?sa) (size_min ?min) (limit ?lm)
			(fishing_type ?tp) (fishing_location ?loc) (best_trip $?dn) (desireability ?ds ))
 (date (dateint ?today))
 (test ( openseason ?open ?close ?today))
  =>
  (assert (menu (species ?sp) (habitat $?hb) (seasonal_availability ?sa) (size_min ?min) (limit ?lm)
			(fishing_type ?tp) (fishing_location ?loc) (best_trip $?dn) (desireability ?ds ) (bonus OFF)))
  (printout t ?sp " is open now. They are currently at " ?hb  " and best fished during the " ?sa  " season. " crlf )
  (printout t "We can fish for them by " ?tp " fishing " ?loc ". " crlf   )
  (printout t "You can have " ?lm  " fish and it is best to fish for them during the " $?dn " trip(s). " crlf )
  (printout t "On a scale of 1-5 they are desireable: " ?ds crlf crlf )
)

(defrule testrule 
 (menu (seasonal_availability $sa3))
 =>
 (printout t  "==> " $sa3 crlf)
 )

(FISH::defrule goodseason
 ?f1 <- (menu (species ?sp) (habitat $?hb) (seasonal_availability ?sa) (size_min ?min) (limit ?lm)
        (fishing_type ?tp) (fishing_location ?loc) (best_trip $?dn) (desireability ?ds )(bonus OFF))
  (date (dateint ?today))
  (season ( description ?sa )(start_date ?open) (end_date ?close))
 (test ( openseason ?open ?close ?today))
 =>
;  (printout t "YOOO!! " ?sp " is open now. They are currently at " ?hb  " and best fished during the " ?sa  " season. " crlf )
;  (printout t "We can fish for them by " ?tp " fishing " ?loc ". " crlf   )
;  (printout t "You can have " ?lm  " fish and it is best to fish for them during the " $?dn " trip(s). " crlf )
;  (printout t "On a scale of 1-5 they are desireable: " ?ds crlf crlf )
; (printout t "What is ?ds => " ?ds crlf crlf )
 (bind ?ds (+ ?ds 2)) ; GOOD SEASON MORE DESIRABLE BY 2
 (retract ?f1)
 (assert (menu (species ?sp) (habitat $?hb) (seasonal_availability ?sa) (size_min ?min) (limit ?lm)
                        (fishing_type ?tp) (fishing_location ?loc) (best_trip ?dn) (desireability ?ds ) (bonus ON) )
 )
  (printout t "Oh we are in Season - even better. This is " ?sa " SEASON"  crlf )
  (printout t ?sp " is open now. They are currently at " ?hb  " and best fished NOW!. " crlf )
  (printout t "We can fish for them by " ?tp " fishing " ?loc ". " crlf   )
  (printout t "You can have " ?lm  " fish and it is best to fish for them during the " $?dn " trip(s). " crlf )
  (printout t "and they are now at desireable level: " ?ds crlf crlf )
) 

	
(deftemplate WEATHER::acceptable_condition ;these are viable non-species specific fishing options based on weather
	(slot locale  (type SYMBOL )(allowed-symbols cancel inshore offshore open_ocean) )
	(slot fsh_type  (type SYMBOL )(allowed-symbols bottom game jigging wreck) )
)

(deftemplate WEATHER::wave_height
	(slot height (type NUMBER ))
)
(deftemplate WEATHER::swells
	(slot seconds (type NUMBER ))
)
(deftemplate WEATHER::wind_speed
	(slot speed (type NUMBER ))
	(slot dir (type SYMBOL )(allowed-symbols EAST WEST NORTH SOUTH ))
)
(defrule WEATHER::WAVEHEIGHT 
	=>
	(printout t "What is the expected Wave Heights today =>" crlf )
	(bind ?height (read))
	(assert (wave_height ( height ?height )))
)

(defrule WEATHER::SWELL
	=>
	(printout t "What Swell Period in Seconds =>" crlf )
	(bind ?sec (read))
	(assert (swells( seconds ?sec )))
)

(defrule WIND
	=>
	(printout t "What is the expected wind speed in knots =>" crlf  )
	(bind ?speed (read))
	(printout t "From what direction is the wind from? (NORTH SOUTH EAST or WEST) =>" crlf  )
	(bind ?direction (read))
	(assert (wind_speed (speed ?speed)(dir ?direction))) 
)
	
(defrule CANCEL 
		(or (wave_height (height ?ht&:(>= ?ht 10)))
			(wind_speed (speed ?sp&:(>= ?sp 30)))
			(wind_speed (speed ?sp&:(>= ?sp 20)) (dir EAST))
			(swells (seconds ?sec&:(>= ?sec 6)))
		)
=>
	(assert (acceptable_condition (locale cancel)))
	(printout t "Weather is too harsh to fish" crlf )
	(exit)
)
; I have ovewrlapping decisions and a single outcome => need a decision tree might be better

(defrule HIGHWINDS
 	(wind_speed (speed ?sp&:(and (>= ?sp 20 ) (< ?sp 30 ))) (dir ~EAST))
 => 
	(assert (acceptable_condition (locale inshore) (fsh_type bottom)))
) 

(defrule MODERATEWINDS
 (wind_speed (speed ?sp&:(and(>= ?sp 15) (< ?sp 20))) (dir WEST))
 =>
  (assert (acceptable_condition (locale offshore) (fsh_type bottom)))
  (assert (acceptable_condition(locale inshore) (fsh_type bottom)))
  (assert (acceptable_condition(locale inshore) (fsh_type game)))
  (assert (acceptable_condition(locale inshore) (fsh_type jigging)))
  (assert (acceptable_condition(locale inshore) (fsh_type wreck)))
 )

(defrule MODERATE_HARSH_WINDS
 (wind_speed (speed ?sp&:(and(>= ?sp 15) (< ?sp 20))) (dir ~WEST))
 =>
  (assert (acceptable_condition(locale inshore) (fsh_type game)))
  (assert (acceptable_condition(locale inshore) (fsh_type bottom)))
) 

(defrule CALMSEAS
 (wind_speed (speed ?sp&:(< ?sp 15)))
 =>
  (assert (acceptable_condition (locale offshore) (fsh_type bottom)))
  (assert (acceptable_condition (locale offshore) (fsh_type game)))
  (assert (acceptable_condition (locale offshore) (fsh_type jigging)))
  (assert (acceptable_condition (locale offshore) (fsh_type wreck)))
  (assert (acceptable_condition (locale inshore) (fsh_type bottom)))
  (assert (acceptable_condition (locale inshore) (fsh_type game)))
  (assert (acceptable_condition (locale inshore) (fsh_type jigging)))
  (assert (acceptable_condition (locale inshore) (fsh_type wreck)))
  (assert (acceptable_condition (locale open_ocean) (fsh_type bottom)))
  (assert (acceptable_condition (locale open_ocean) (fsh_type game)))
  (assert (acceptable_condition (locale open_ocean) (fsh_type jigging)))
  (assert (acceptable_condition (locale open_ocean) (fsh_type wreck)))
)

(deftemplate all_trips 
	 (slot location ) (slot method) (multislot species) (slot desireability) (slot check (type INTEGER))
)
(deftemplate table
 	(slot location) (slot method)
	)

(deftemplate trips 
	 (slot location ) (slot method) (multislot species) (slot desireability) 
)

(defrule all_possible_trips
	 ?f1 <- (menu (species ?spec) (habitat $?hab) (size_min ?min) 
		 (limit ?lim) (fishing_type ?ft) (fishing_location ?loc) (best_trip $?) (desireability ?ds))
	 ?f2 <- (acceptable_condition (locale ?loc) (fsh_type ?ft))

	 =>
	 (assert (all_trips (location ?loc) (method ?ft) (species ?spec) (desireability ?ds) (check 1)))
;	 (format t "Trip: %-15s Method: %-15s  Species: %-18s Profit Points: %4d" ?loc ?ft ?spec ?ds)
;	 (printout t crlf )
)
   
(assert (sum 0))
(assert (spec_tmp))
	
(defrule trips
			?f1 <- (all_trips (location ?loc) (method ?ft) (species $?spec) (desireability ?ds) (check 1))
			?f2 <- (spec_tmp $?spec_tmp)
			?f3 <- (sum ?sum)
			=>
			 (modify ?f1 (check 0))
			 (retract ?f2)
			(assert (spec_tmp (create$ ?spec_tmp ?spec) ))
			(retract ?f3) 
			(assert (sum =(+ ?sum ?ds ) ))
)

(defrule make_table
 	?f1 <- (all_trips (location ?s_loc) (method ?s_ft) )
	=>
	(assert (table (location ?s_loc) (method ?s_ft) ))
)

(defrule eval_function_new
 ?f1 <- (table (location ?s_loc) (method ?s_ft))
 ?f2 <- (all_trips (location ?s_loc) (method ?s_ft) (species $?spec) (desireability ?ds) (check 0))
; (bind $?species (create$ ?species ?spec))
; (bind ?sum (create$ ?sum ?ds))
 (not (exists (trips (location ?s_loc ) (method ?s_ft))))
 =>
 (modify ?f2 (check -1));
 (assert (trips (location ?s_loc ) (method ?s_ft) (species ?spec ) (desireability ?ds)))
 )

(defrule eval_function_cont
 ?f3 <- (trips (location ?s_loc ) (method ?s_ft) (species $?species ) (desireability ?sum))
 ?f1 <- (table (location ?s_loc) (method ?s_ft))
 ?f2 <- (all_trips (location ?s_loc) (method ?s_ft) (species $?spec) (desireability ?ds) (check 0))
; (bind $?species =(create$ ?species ?spec))
; (bind ?sum =(+ ?sum ?ds))
; (exists (trips (location ?s_loc) (method s_ft )))
 =>
 (modify ?f2 (check -1))
 (retract ?f3)
 (assert (trips (location ?s_loc ) (method ?s_ft) (species =(create$ ?species ?spec )) (desireability =(+ ?sum  ?ds))))
 )
 

 (defrule final_report 
   (trips (location ?s_loc) (method ?s_ft) (species $?species) (desireability ?ds))
	=>
 (format t "Trip: %-15s Method: %-15s  Species: %-45s Profit Points: %4d" ?s_loc ?s_ft (implode$ ?species) ?ds)
 (printout t crlf )
 )
 

